<?php

/****************************************************************************

	Copyright (C) 2017-2020  Sendatel S.A.

	This file is part of TUIDD PHP SDK.
	TUIDD PHP SDK is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	TUIDD PHP SDK is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with TUIDD PHP SDK.  If not, see <http://www.gnu.org/licenses/>.

****************************************************************************/


require_once('libs/tuidd_lib/client/TuiddClient.php');

$sess_ = TuiddClient::blockStart('sdk-session');

/**
 * Discovery
 * Se obtienen urls para auth, pedido de token y consumo de servicios
 */
if (( isset( $_GET['init'] ) && $_GET['init'] == 'login' ) && (isset($_GET['action']) && $_GET['action'] == 'discovery'))
{
	$tuidd = TuiddClient::getSingleton();
   	$url = $tuidd->discovery();
    TuiddClient::blockEnd($sess_);
	echo json_encode( array('request_url' => $url));
}


/**
 * Se extrae el codigo que viene luego de la autorizacion
 * Se lo utiliza para pedir Token de acceso y la Informacion de usuario.
 * Finalmente se llama a la funcion de callback.
 */
if (isset($_GET['code']) && $_GET['code'] != '')
{
	$code = $_GET['code'];
	$tuidd = TuiddClient::getSingleton();

    if ( !$tuidd->hasDiscoveryUrls() ) {
        $tuidd->discovery();
    }

	// Luego de obtener el codigo se pide el token
    // la informacion de usuario
    // se llama a la funcion de callback
	try {
		$tuidd->getToken($code);
		$tuidd->requestUserInfo();

		$arrStates = $tuidd->readSdkState($_GET['state']);
		$state = $arrStates['client_state'];
		$sdkState = $arrStates['sdk_state'];
		if( strtolower( $sdkState ) == 'binding' )
		{
			$return =json_encode( [ 'status' => 'SUCCESS' ] );
			if ( $tuidd->isDebugging( ) )
			{
				syslog(LOG_INFO, "TUIDD SDK :: ". __METHOD__ ." :: BINDING SUCCESS: ");
			}
			echo $return;
		}
		else
		{
			if ( $tuidd->isDebugging(   ) )
			{
				syslog(LOG_INFO, "TUIDD SDK :: ". __METHOD__ ." :: ACCEDIENDO AL SITIO: ");
			}
			$tuidd->cfReceive();	
		}
	}
	catch (Exception $e) {
    	echo "Tuidd Error: ", $e->getMessage();
        // Si se detecto error se redirigue a Home page
		$home_page_url = $tuidd->getHomePageURL();
  		header('refresh:5; ' . $home_page_url);
	}
    TuiddClient::blockEnd($sess_);
}


/***
 * Loadbutton
 * Carga y dibuja el boton de tuidd.
 * Ademas forma la URL de request y el texto del boton en el idioma 
 * correspondiente.
 */
if( isset( $_GET['action'] ) && $_GET['action'] == 'loadbutton' )
{
	$tuidd = TuiddClient::getSingleton();

	$lang = $_SESSION['lang-'.$tuidd->getClientID()];
	$locale = $_SESSION['locale-'.$tuidd->getClientID()];

    // Se genera Request Uri para luego pasarla al boton
    // con la informacion necesaria
	$client_id      = $tuidd->getClientID();
	$scope          = $tuidd->getScope();
	$acr_values     = $tuidd->getSecurityLevel();
	$redirect_uri   = $tuidd->getRedirectURI();
	$container_div  = $tuidd->getContainerDiv();
	$tuidd_sdk_path = $tuidd->getTuiddSDKPath();

    if ( $tuidd->isDebugging() )
    {
        syslog(LOG_INFO, "". basename(__FILE__, ".php") ." :: callback_file :: " . $tuidd->getCallbackFunctionFile());
        syslog(LOG_INFO, "". basename(__FILE__, ".php") ." :: callback state func :: " . $tuidd->getStateNonceCallback());
    }

    // Se llama a la funcion de callback establecida en archivo de config
    // para generar state y nonce
    $tuidd->generateStateAndNonce();

    // Se refresca la sesion
    $tuidd = TuiddClient::getSingleton(); 
    // $state = $tuidd->getState();
    $state = $tuidd->generateSdkState($tuidd->getState());
    $nonce = $tuidd->getNonce();

	$response_type = 'code';
	$display = 'page';


    // PATCH STATE
    require_once($tuidd->getCallbackFunctionFile());
    if (function_exists('beforeButtonDrawCallback'))
    {
        list($state, $nonce) = beforeButtonDrawCallback($redirect_uri, $client_id);
    }

    // // PATCH STATE
    // $state     = 'access';
    // $action    = 'create_state';
		
    // $url = $redirect_uri;
    // $data = 'action='.$action.'&client_id='.$client_id.'&state='.$state;
    // $options = array(
    //     'method' => 'GET',
    //     //'data' => drupal_http_build_query($data),
    //     'timeout' => 15,
    //     'headers' => array(),
    //     'context' => stream_context_create( 
    //         array( 'ssl' => array( 'verify_peer' => FALSE ,
    //                                'verify_peer_name' => FALSE,
    //                                'allow_self_signed' => TRUE
    //         ) 

    //         )
    //     )
    // );
    // $r = drupal_http_request($redirect_uri."?".$data, $options);
    // syslog(LOG_WARNING, " TUIDD REQUEST :: ". print_r($r, true));    

    // if (isset($r->data))
    // {
    //     $rd = json_decode($r->data, true);
    //     $state = $rd['state'];
    //     $nonce = $rd['nonce'];
				
    // }		
    // // FIN PATCH STATE

    // concatenacion de variables
	$urlRequest = '{request_url}' . 
                '?client_id=' . $client_id . 
                '&response_type=' . $response_type . 
                '&scope=' . $scope .
                '&redirect_uri=' . $redirect_uri .
                '&acr_values=' . $acr_values;

    if ( $state != '') {
        $urlRequest = $urlRequest . '&state=' . $state;
    } else {
        $urlRequest = $urlRequest . '&state=state';
        $tuidd->setState('state');
    }
    if ( $nonce != '') {
        $urlRequest = $urlRequest . '&nonce=' . $nonce;
    } else {
        $urlRequest = $urlRequest . '&nonce=nonce';
        $tuidd->setNonce('nonce');
    }
    $urlRequest = $urlRequest .
				'&display=' . $display .
				'&ui_locales=' . $locale;
	
    // Se hace un request para obtener el html del boton
    $sdk_wrapper_file = $tuidd->getTuiddContentURL() . 'html/sdk-wrapper';
	$wrapper = curl_file_get_contents( $sdk_wrapper_file );
    if ($wrapper === FALSE) {
        echo json_encode( array(
			'error' => 'No se pudo cargar boton. Fallo el request a '.$sdk_wrapper_file
		));
    }
    else {
        // se reemplazan variables por valores definitivos, URL request,
        // cadenas traducidas del boton, etc.
        $wrapper = str_replace("{request}", $urlRequest, $wrapper);
        $wrapper = str_replace("{msisdn}", '', $wrapper);
        $wrapper = str_replace("{login_without_pass}", $lang['login_without_pass'], $wrapper);
        $wrapper = str_replace("{phone_number}", $lang['phone_number'], $wrapper);
        $wrapper = str_replace("{login_button}", $lang['login_button'], $wrapper);
        $wrapper = str_replace("{have_no_account}", '', $wrapper);
        $wrapper = str_replace("{security}", $lang['security'], $wrapper);
        $wrapper = str_replace("{more_security}", $lang['more_security'], $wrapper);
        $wrapper = str_replace("{privacy}", $lang['privacy'], $wrapper);
        $wrapper = str_replace("{more_info}", $lang['more_info'], $wrapper);
        $wrapper = str_replace("{tuidd_sdk_path}", $tuidd->getTuiddContentURL() . '/css/', $wrapper);
        $wrapper = str_replace("{convinient}", $lang['convinient'], $wrapper);
        $wrapper = str_replace("{one_way_login}", $lang['one_way_login'], $wrapper);
        $wrapper = str_replace("{create_account}", $lang['create_account'], $wrapper);
        $wrapper = str_replace("{personal_info}", $lang['personal_info'], $wrapper);
        $wrapper = str_replace("{advantages}", $lang['advantages'], $wrapper);
        $wrapper = str_replace('\"', '"', $wrapper);

        echo json_encode( array(
			'wrapper' => $wrapper,
			'container_div' => $container_div
		));
    }
    TuiddClient::blockEnd($sess_);
}


function curl_file_get_contents($file)
{
	if (function_exists('curl_version'))
	{
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $file);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);

        $content = curl_exec($curl);
        curl_close($curl);
	}
	else if (file_get_contents(__FILE__) && ini_get('allow_url_fopen'))
	{
        $arrContextOptions=array(
            "ssl"=>array(
                "verify_peer"=>false,
                "verify_peer_name"=>false,
            ),
        ); 
        $content = file_get_contents($file, false, stream_context_create($arrContextOptions));
	}
	else
	{
        echo 'You have neither cUrl installed nor allow_url_fopen activated. Please setup one of those!';
	}
    return $content;
}


/**
 * Error
 * Se detecta errores en la autenticacion.
 * Se redirecciona al home page.
 */
if (isset($_GET['error']) && $_GET['error'] != '')
{
	$tuidd = TuiddClient::getSingleton();
	$home_page_url = $tuidd->getHomePageURL();
    TuiddClient::blockEnd($sess_);
	header('Location: ' . $home_page_url . '?error=' . $_GET['error']);
}


/***
 * Peticion de state y nonce
 */
if( isset( $_GET['action'] ) && $_GET['action'] == 'create_state' )
{
    $tuidd = TuiddClient::getSingleton();
    // FIXME: Configurar la functionCallback de state y nonce y llamaren este punto. JN 2018-06-28
    $state = $tuidd->getState();
    $nonce = $tuidd->getNonce();
    if ($state == '' )
        $tuidd->setState("state");
    if ($nonce == '' )
        $tuidd->setNonce("nonce");

    if ( isset($_GET['client_id'] ) && $_GET['client_id'] == $tuidd->getClientID() )
    {
	    if ( isset( $_GET['state'] ) && ($_GET['state'] == 'binding' || $_GET['state'] == 'access'  ))
	    {

	    	$state = $tuidd->generateSdkState($tuidd->getState(), $_GET['state']);

            // Se obtiene informacion necesaria desde session
            // para responder al pedido 
            $ret['client_id'] = $tuidd->getClientId();
            $ret['scope'] = $tuidd->getScope();
            $ret['acr_value'] = $tuidd->getSecurityLevel();
            $ret['redirect_uri'] = $tuidd->getRedirectURI();
            $ret['response_type'] = 'code';
            $ret['display'] = 'page';
            $ret['state'] = utf8_encode($state);
            $ret['nonce'] = $tuidd->getNonce();

            echo json_encode($ret);
        }
    }
    else
    {
        $ret['error'] = 'Wrong client_id';
        echo json_encode( $ret );
    }
    TuiddClient::blockEnd($sess_);
}
?>
