<?php

/****************************************************************************

	tuidd's PHP SDK version v2.0.0

	Copyright (C) 2017-2020  Sendatel S.A.

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

/****************************************************************************/

require_once('TuiddException.php');
require_once('TuiddClient.php');

session_name('sdk-session');
session_start();

class TuiddNotificationHandler
{
	private $callback_function;
	private $callback_function_file;
    private $client_id;
        
	private function __construct()
	{
        $root_dir = dirname( dirname( dirname( dirname(__FILE__) ) ) ) ;
        $configs = parse_ini_file( $root_dir . '/conf/tuidd.conf' ); 
		$this->callback_function_file = $configs['callback_function_file'];
		$this->notif_callback_function = $configs['services_notif_callback_function'];
        $this->clientId = $configs['client_id'];
	}

	/**
	 * Gets Singleton's instance of TuiddClient 
	 * @param  none
	 * @return [type]              [description]
	 */
	public static function getSingleton()
	{
		if ( !isset($this) )
        {
        	$root_dir = dirname( dirname( dirname( dirname(__FILE__) ) ) ) ;
			$configs = parse_ini_file( $root_dir . '/conf/tuidd.conf' ); 
        	$clientId = $configs['client_id'];
        }
        else {
			$clientId = $this->client_id;
        }	
		
		if ( $_SESSION['ns'.$clientId] == '' ) {
			return new self();
		}
		else
    	{
            if( $_SESSION['ns'.$clientId] instanceof TuiddNotificationHandler ) {
                return $_SESSION['ns'.$clientId];
            } else {
                $ret = new self();
                $ret = unserialize($_SESSION['ns'.$clientId]);
                return $ret;
            } 
    	}
	}

	public function processNotification( $notification )
	{
		if ( !empty($this->callback_function_file) && $this->callback_function_file != '' 
             && !empty($this->notif_callback_function) && $this->notif_callback_function != '')
		{
			require_once( $this->callback_function_file );
			call_user_func( $this->notif_callback_function, $notification );
		}
	}
}

